import meteva

def interpolation_linear(grd, grid):
    members = grd["member"]
    num = len(members)
    grid_old = meteva.base.get_grid_of_data(grd)
    grid_new = meteva.base.grid(grid.glon,grid.glat,members=members)
    grd_new = meteva.base.grid_data(grid_new)
    grid_xy = meteva.base.grid(grid_old.glon,grid_old.glat)
    for m in range(num):
        member = members[m]
        grd0 = grd.loc[dict(member=member)]
        grd1 = meteva.base.grid_data(grid_xy)
        grd1.values[0,0,0,0,:,:] = grd0.values.squeeze()
        #print(grd1)
        #print(grid)
        grd2 = meteva.base.function.gxy_gxy.interpolation_linear(grd1,grid)
        #print(grd2)
        grd_new.values[m,:,:,:,:,:] = grd2.values

    return grd_new


import meteva
from meteva.base.tool.math_tools import lon_lat_to_cartesian
from scipy.spatial import cKDTree
import numpy as np
import copy

def get_nearby_sta_index_ensemble(sta_to,nearNum = 100,sta_from = None,drop_frist = False):
    if(sta_to is None):
        return None
    if(sta_from is None):
        sta_from = copy.deepcopy(sta_to)
    xyz_sta0 = lon_lat_to_cartesian(sta_to['lon'].values[:], sta_to['lat'].values[:],R = meteva.base.basicdata.ER)
    xyz_sta1 = lon_lat_to_cartesian(sta_from['lon'].values[:], sta_from['lat'].values[:],R = meteva.base.basicdata.ER)
    tree = cKDTree(xyz_sta0)
    _,indexs = tree.query(xyz_sta1, k=nearNum)
    sta_ensemble = sta_to[meteva.base.get_coord_names()]
    for i in range(nearNum):
        data_name = "data" + str(i)
        sta_ensemble[data_name] = indexs[:,i]
    if drop_frist:
        sta_ensemble = sta_ensemble.drop(columns=['data0'])
    return sta_ensemble

def get_nearby_sta_id_ensemble(sta_to,nearNum = 100,sta_from = None,drop_frist = False):
    if(sta_to is None):
        return None
    if(sta_from is None):
        sta_from = copy.deepcopy(sta_to)
    xyz_sta0 = lon_lat_to_cartesian(sta_to['lon'].values[:], sta_to['lat'].values[:],R = meteva.base.basicdata.ER)
    xyz_sta1 = lon_lat_to_cartesian(sta_from['lon'].values[:], sta_from['lat'].values[:],R = meteva.base.basicdata.ER)
    tree = cKDTree(xyz_sta0)
    _,indexs = tree.query(xyz_sta1, k=nearNum)
    input_dat = sta_from.ix[:, 'id'].values
    sta_ensemble = sta_to[meteva.base.get_coord_names()]
    for i in range(nearNum):
        data_name = "data" + str(i)
        sta_ensemble[data_name] = input_dat[indexs[:,i]]
    if drop_frist:
        sta_ensemble = sta_ensemble.drop(columns=['data0'])
    return sta_ensemble

def get_nearby_sta_value_ensemble(sta_to,nearNum = 100,sta_from = None,drop_frist = False):
    if(sta_to is None):
        return None
    if(sta_from is None):
        sta_from = copy.deepcopy(sta_to)
    xyz_sta0 = lon_lat_to_cartesian(sta_to['lon'].values[:], sta_to['lat'].values[:],R = meteva.base.basicdata.ER)
    xyz_sta1 = lon_lat_to_cartesian(sta_from['lon'].values[:], sta_from['lat'].values[:],R = meteva.base.basicdata.ER)
    tree = cKDTree(xyz_sta0)
    _,indexs = tree.query(xyz_sta1, k=nearNum)
    data_name = meteva.base.get_data_names(sta_from)[0]
    input_dat = sta_from[data_name].values
    sta_ensemble = sta_to[meteva.base.get_coord_names()]
    for i in range(nearNum):
        data_name = "data" + str(i)
        sta_ensemble[data_name] = input_dat[indexs[:,i]]
    if drop_frist:
        sta_ensemble = sta_ensemble.drop(columns=['data0'])
    return sta_ensemble

def get_nearby_sta_dis_ensemble(sta_to,nearNum = 100,sta_from = None,drop_frist = False):
    if(sta_to is None):
        return None
    if(sta_from is None):
        sta_from = copy.deepcopy(sta_to)
    xyz_sta0 = lon_lat_to_cartesian(sta_to['lon'].values[:], sta_to['lat'].values[:],R = meteva.base.basicdata.ER)
    xyz_sta1 = lon_lat_to_cartesian(sta_from['lon'].values[:], sta_from['lat'].values[:],R = meteva.base.basicdata.ER)
    tree = cKDTree(xyz_sta0)
    d,_ = tree.query(xyz_sta1, k=nearNum)
    sta_ensemble = sta_to[meteva.base.get_coord_names()]
    for i in range(nearNum):
        data_name = "data" + str(i)
        sta_ensemble[data_name] = d[:,i]
    if drop_frist:
        sta_ensemble = sta_ensemble.drop(columns=['data0'])
    return sta_ensemble






#一些功能性的模块依次是格点到站点插值，格点到格点插值，获取格点数据，获取站点数据
from . import gxy_sxy
from . import gxy_gxy
from . import get_from_grid_data
from . import get_from_sta_data
from . import gxym_gxy
from . import put_into_grid_data
from . import put_into_sta_data
from . import sxy_sxy
from . import sxy_gxy
from . import sxy_sxym
from . import sxym_sxy
from . import gxy_gxym
from . import gxym_gxym
from . import sxy_gxym



import meteva
import copy
import pandas as pd
import numpy as np

def sta_between_value_range(sta,start_value,end_value,start_open = False,end_open = False):
    data_name = meteva.base.basicdata.get_data_names(sta)[0]
    if start_open:
        if end_open:
            sta1 = sta.loc[(sta[data_name] > start_value) & (sta[data_name] < end_value)]
        else:
            sta1 = sta.loc[(sta[data_name] > start_value) & (sta[data_name] <= end_value)]
    else:
        if end_open:
            sta1 = sta.loc[(sta[data_name] >= start_value) & (sta[data_name] < end_value)]
        else:
            sta1 = sta.loc[(sta[data_name] >= start_value) & (sta[data_name] <= end_value)]
    return sta1
#为站点数据中dataframe重新赋列名
def sta_of_name(sta,data_name):
    columns = ['level', 'time', 'dtime', 'id', 'lon', 'lat', 'alt',data_name]
    sta1 = sta[columns]
    return sta1

#为拥有多元素值的站点数据，在最后依次增加要素值的列表名
def sta_in_name_list(sta,data_name_list):
    columns = ['level', 'time', 'dtime', 'id', 'lon', 'lat', 'alt'] + data_name_list
    sta1 = sta[columns]
    return sta1

#为站点信息重新赋level名称
def sta_of_level(sta,level):
    sta1 = sta.loc[sta['level']==level]
    return sta1

#为拥有多level层的站点数据，依次增加level层所表示的list列表
def sta_in_level_list(sta,level_list):
    sta1 = sta.loc[sta['level'].isin(level_list)]
    return sta1

#为站点信息重新赋id名称
def sta_of_id(sta,id):
    sta1 = sta.loc[sta['id']==id]
    return sta1

#为拥有多id的站点数据，依次增加id所表示的list列表
def sta_in_id_list(sta,id_list):
    sta1 = sta.loc[sta['id'].isin(id_list)]
    return sta1

#为站点信息重新赋time层名称
def sta_of_time(sta,time):
    time1= meteva.base.tool.time_tools.all_type_time_to_time64(time)
    sta1 = sta.loc[sta['time'] == time1]
    return sta1


#为拥有多time层的站点数据，依次增加time层所表示的list列表
def sta_in_time_list(sta,time_list):
    time_list1 = []
    for time0 in time_list:
        time_list1.append(meteva.base.tool.time_tools.all_type_time_to_time64(time0))
    sta1 = sta.loc[sta['time'].isin(time_list1)]
    return sta1

#为拥有多year的站点数据，依次增加year所表示的list列表
def sta_in_year_list(sta,year_list):
    years = sta['time'].map(lambda x: x.year)
    sta1 = sta.loc[years.isin(year_list)]
    return sta1


#为拥有多month的站点数据，依次增加month所表示的list列表
def sta_in_month_list(sta,month_list):
    months = sta['time'].map(lambda x: x.month)
    sta1 = sta.loc[months.isin(month_list)]
    return sta1

#为拥有多xun的站点数据，依次增加xun所表示的list列表
def sta_in_xun_list(sta,xun_list):
    mons = sta['time'].map(lambda x: x.month).values.astype(np.int16)
    days = sta['time'].map(lambda y: y.day).values
    xuns = np.ceil(days / 10).astype(np.int16)
    xuns[xuns>3] = 3
    xuns += (mons - 1) * 3
    xuns = pd.Series(xuns)
    sta1 = sta.loc[xuns.isin(xun_list)]
    return sta1

#为拥有多hou的站点数据，依次增加hou所表示的list列表
def sta_in_hou_list(sta,hou_list):
    mons = sta['time'].map(lambda x: x.month).values.astype(np.int16)
    days = sta['time'].map(lambda y: y.day).values
    hous = np.ceil(days / 5).astype(np.int16)
    hous[hous>6] = 6
    hous += (mons - 1) * 6
    hous = pd.Series(hous)
    sta1 = sta.loc[hous.isin(hou_list)]
    return sta1

#为拥有多day的站点数据，依次增加day所表示的list列表
def sta_in_day_list(sta,day_list):
    days = sta['time'].map(lambda x: x.dayofyear)
    sta1 = sta.loc[days.isin(day_list)]
    return sta1

#为拥有多hour的站点数据，依次增加hour所表示的list列表
def sta_in_hour_list(sta,hour_list):
    hours = sta['time'].map(lambda x: x.hour)
    sta1 = sta.loc[hours.isin(hour_list)]
    return sta1

def sta_between_time_range(sta,start_time,end_time,dtime = None):
    #空函数，待完善
    return sta

#为站点信息重新赋dtime层名称
def sta_of_dtime(sta,dtime):
    sta1 = sta.loc[sta['dtime']==dtime]
    return sta1

#为拥有多dtime的站点数据，依次增加dtime所表示的list列表
def sta_in_dtime_list(sta,dtime_list):
    sta1 = sta.loc[sta['dtime'].isin(dtime_list)]
    return sta1

#为拥有多dday的站点数据，依次增加dday所表示的list列表
def sta_in_dday_list(sta,dday_list):
    seconds = sta['dtime'].map(lambda x: x/np.timedelta64(1, 's')).values
    days = np.ceil(seconds/(24*3600))
    days = pd.Series(days)
    sta1 = sta.loc[days.isin(dday_list)]
    return sta1

#为拥有多dhour的站点数据，依次增加dhour所表示的list列表
def sta_in_dhour_list(sta,dhour_list):
    seconds = sta['dtime'].map(lambda x: x/np.timedelta64(1, 's')).values
    hours = np.ceil(seconds/(3600)).astype(np.int16)
    hours = pd.Series(hours)
    sta1 = sta.loc[hours.isin(dhour_list)]
    return sta1

#为拥有多dminute的站点数据，依次增加minute所表示的list列表
def sta_in_dminute_list(sta,dminute_list):
    seconds = sta['dtime'].map(lambda x: x/np.timedelta64(1, 's')).values
    minutes = np.ceil(seconds/(60))
    minutes = pd.Series(minutes)
    sta1 = sta.loc[minutes.isin(dminute_list)]
    return sta1

#返回的dtime在start_dtime和end_dtime之间
def sta_between_dtime_range(sta,start_dtime,end_dtime,d_dtime):
    #空函数，待完善
    return sta


#返回的lon在slon和elon之间
def sta_between_lon_range(sta,slon,elon):
    sta1 = sta.loc[(sta['lon']>=slon) & (sta['lon']<= elon)]
    return sta1


#返回的lat在slat和elat之间
def sta_between_lat_range(sta,slat,elat):
    sta1 = sta.loc[(sta['lat']>=slat) & (sta['lat']<= elat)]
    return sta1

#返回的alt在salt和ealt之间
def sta_between_alt_range(sta,salt,ealt):
    sta1 = sta.loc[(sta['alt']>=salt) & (sta['alt']<= ealt)]
    return sta1

#返回站点经纬度正好落在格点中的站点信息
def sta_in_grid_xy(sta,grid):
    sta1 = sta_between_lon_range(sta,grid.slon,grid.elon)
    sta2 = sta_between_lat_range(sta1,grid.slat,grid.elat)
    return sta2

#返回站点参数字典列表
def get_by_para_dict(sta,para_dict):
    '''
    para_dict 应具备如下样式
    para_dict = {'level' :[850,700],  列表形式
                'time' : ['2018010108','2018010208'], 列表形式
                'dtime':['24h','36h']，列表形式
                'id':[54511]， 列表形式
                'lon': [70,140], 闭区间
                'lat' :[10,60], 闭区间
                'alt’: [ 0,9999] 闭区间
    }
    '''
    sta1 = sta_in_level_list(sta,para_dict['level'])
    sta1 = sta_in_time_list(sta1,para_dict['time'])
    sta1 = sta_in_dtime_list(sta1, para_dict['dtime'])
    sta1 = sta_in_id_list(sta1, para_dict['id'])
    sta1 = sta_between_lon_range(sta1,para_dict['lon'][0],para_dict['lon'][1])
    sta1 = sta_between_lat_range(sta1, para_dict['lat'][0], para_dict['lat'][1])
    sta1 = sta_between_alt_range(sta1, para_dict['alt'][0], para_dict['alt'][1])
    return sta1




import numpy as np
import math
import meteva

def put_uv_into_wind(u,v):
    grid0 = meteva.base.get_grid_of_data(u)
    grid1 = meteva.base.grid(grid0.glon,grid0.glat,grid0.gtime,
                                              dtime_list= grid0.dtimes,level_list=grid0.levels,member_list=["u","v"])
    wind = meteva.base.grid_data(grid1)
    wind.name = "wind"
    wind.values[0, :, :, :, :, :] = u.values[0, :, :, :, :, :]
    wind.values[1, :, :, :, :, :] = v.values[0, :, :, :, :, :]
    return wind

def get_wind_from_speed_angle(speed,angle):

    speed_v = speed.values.squeeze()
    angle_v = angle.values.squeeze()

    grid0 = meteva.base.get_grid_of_data(speed)
    grid1 = meteva.base.grid(grid0.glon,grid0.glat,grid0.gtime,
                                              dtime_list=grid0.dtimes,level_list=grid0.levels,member_list=["u","v"])
    wind = meteva.base.grid_data(grid1)
    wind.name = "wind"
    wind.values[0, :, :, :, :, :] = speed_v[:, :] * np.cos(angle_v[:, :] * math.pi /180)
    wind.values[1, :, :, :, :, :] = speed_v[:, :] * np.sin(angle_v[:, :] * math.pi /180)
    return wind



import numpy as np
import pandas as pd
import datetime
import re

import meteva_base

def get_time_input_single(gtime): 
    time = meteva_base.all_type_time_to_time64(gtime[0])
    stime = time
    etime = time
    dtime_int = 1
    dtime_type = "h"
    dtimedelta = np.timedelta64(dtime_int, dtime_type)
    times = pd.date_range(stime, etime, freq=str(dtime_int)+dtime_type ).to_pydatetime()
    return times

def get_time_input_three(gtime):  
    num1 =[]
    ## stime etime
    if type(gtime[0]) == str:
        for i in range (0,2):
            num = ''.join([x for x in gtime[i] if x.isdigit()])
            #用户输入2019041910十位字符，后面补全加0000，为14位统一处理
            if len(num) == 4:
                num1.append(num + "0101000000")
            elif len(num) == 6:
                num1.append(num + "01000000")
            elif len(num) == 8:
                num1.append(num + "000000")
            elif len(num) == 10:
                num1.append(num + "0000")
            elif len(num) == 12:
                num1.append(num + "00")
            elif len(num) == 14:
                num1.append(num)
            else:
                print("输入日期有误，请检查！")
            #统一将日期变为datetime类型
        #print(num1)
        stime = datetime.datetime.strptime(num1[0], '%Y%m%d%H%M%S')
        etime = datetime.datetime.strptime(num1[1], '%Y%m%d%H%M%S')
        stime = np.datetime64(stime)
        etime = np.datetime64(etime)
    elif isinstance(gtime[0],np.datetime64):
        stime = gtime[0].astype(datetime.datetime)
        etime = gtime[1].astype(datetime.datetime)
        if isinstance(stime, int):
            stime = datetime.datetime.utcfromtimestamp(stime / 1000000000)
            etime = datetime.datetime.utcfromtimestamp(etime / 1000000000)
        stime = stime
        etime = etime
    else:
        stime = gtime[0]
        etime = gtime[1]
    ## dtime
    if type(gtime[2]) == str:
        dtime_int = re.findall(r"\d+", gtime[2])[0]
        dtime_type = re.findall(r"\D+", gtime[2])[0]
        if dtime_type == 'h':
            dtime_type ="h"
            dtimedelta = np.timedelta64(dtime_int,'h')
        elif dtime_type == 'd':
            dtime_type ="D"
            dtimedelta = np.timedelta64(dtime_int, 'D')
        elif dtime_type == 'm':
            dtime_type ="m"
            dtimedelta = np.timedelta64(dtime_int, 'm')
    elif isinstance(gtime[2],np.timedelta64):
        seconds = int(gtime[2] / np.timedelta64(1, 's'))
        if seconds % 3600 == 0:
            dtime_type = "h"
            dtime_int = int(seconds / 3600)
        else:
            dtime_type = "m"
            dtime_int = int(seconds / 60)
    else:
        dtimedelta = gtime[2]
        seconds = gtime[2].total_seconds()
        if seconds % 3600 == 0:
            dtime_type = "h"
            dtime_int = int(seconds/3600)
        else:
            dtime_type = "m"
            dtime_int = int(seconds / 60)
    times = pd.date_range(stime, etime, freq=str(dtime_int)+dtime_type ).to_pydatetime()
    return times





# import meteva
import meteva_base
import meteva_base as meb
import numpy as np
import pandas as pd
import datetime


def mean_of_sta(sta,used_coords = ["member"],span = 24,equal_weight = False,keep_all = True):
    if not isinstance(used_coords,list):
        used_coords = [used_coords]

    sta1 = meteva_base.not_IV(sta)

    if used_coords == ["member"]:
        sta_mean = sta1.loc[:,meteva_base.get_coord_names()]
        sta_data = sta1[meteva_base.get_stadata_names(sta1)]
        value = sta_data.values
        mean = np.mean(value,axis=1)
        sta_mean['mean'] = mean
        return sta_mean
    elif used_coords == ["time"]:
        times = sta.loc[:, 'time'].values
        times = list(set(times))
        times.sort()
        times = np.array(times)
        dtimes = times[1:] - times[0:-1]
        min_dtime = np.min(dtimes)
        min_dhour = min_dtime / np.timedelta64(1, 'h')

        if span is None:

            rain_ac = meteva_base.in_time_list(sta, [times[0]])
            rain_ac["count_for_add"] = 1
            meteva_base.set_stadata_coords(rain_ac, time=times[-1])
            for i in range(1, len(times)):
                rain01 = meteva_base.in_time_list(sta, times[i])
                rain01["count_for_add"] = 1
                meteva_base.set_stadata_coords(rain01, time=times[-1])
                rain_ac = meteva_base.add_on_level_time_dtime_id(rain_ac, rain01, how="outer", default=0)

            names = meteva_base.get_stadata_names(rain_ac)
            for n in range(len(names)-1):
                rain_ac[names[n]] /= rain_ac[names[-1]]
            rain_ac = meteva_base.in_member_list(rain_ac,names[:-1])
            rain_ac.attrs["valid_time"] = (times[-1] - times[0])/np.timedelta64(1, 'h')
            return rain_ac

        else:
            step = int(round(span/min_dhour))
            sta1["count_for_add"] = 1
            if not equal_weight:
                names = meteva_base.get_stadata_names(sta1)
                rain_ac = sta1.copy()
                for name in names:
                    rain_ac[name] *= 0.5

                for i in range(1,step):
                    rain1 = sta1.copy()
                    rain1["time"] = rain1["time"] + min_dtime * i
                    rain_ac = meteva_base.add_on_level_time_dtime_id(rain_ac, rain1, how="outer",default=0)
                rain1 = sta1.copy()
                rain1["time"] = rain1["time"] + min_dtime * step
                for name in names:
                    rain1[name] *= 0.5
                rain_ac = meteva_base.add_on_level_time_dtime_id(rain_ac, rain1, how="outer", default=0)

            else:
                rain_ac = None
                for i in range(step):
                    rain1 = sta1.copy()
                    rain1["time"] = rain1["time"] + min_dtime * i
                    rain_ac = meteva_base.add_on_level_time_dtime_id(rain_ac, rain1, how="outer",default=0)

            names = meteva_base.get_stadata_names(rain_ac)
            for n in range(len(names)-1):
                rain_ac[names[n]] /= rain_ac[names[-1]]
            rain_ac = meteva_base.in_member_list(rain_ac,names[:-1])
            rain_ac = meteva_base.between_time_range(rain_ac, times[0], times[-1])  # 删除时效小于range的部分

            if not keep_all:
                dtimes = times[:] - times[-1]
                dh = (dtimes / min_dtime).astype(np.int32)
                new_times = times[dh % step == 0]
                rain_ac = meteva_base.in_time_list(rain_ac, new_times)

            rain_ac.attrs["valid_time"] = span
            return rain_ac

    elif used_coords ==["dtime"]:
        if span is None:
            print("if used_coords == [dtime], span must be int of float bigger than 0 ")

        dtimes = sta.loc[:, 'dtime'].values
        dtimes = list(set(dtimes))
        dtimes.sort()
        dtimes = np.array(dtimes)

        dhour_unit = dtimes[0]
        if dhour_unit == 0:
            dhour_unit = dtimes[1]
        step = int(round(span/dhour_unit))

        if equal_weight:
            sta1["count_for_add"] = 1
            names = meteva_base.get_stadata_names(sta1)
            rain_ac = sta1.copy()
            for i in range(1,step):
                rain1 = sta1.copy()
                rain1["dtime"] = rain1["dtime"] + dhour_unit * i
                rain_ac = meteva_base.add_on_level_time_dtime_id(rain_ac, rain1,how="outer",default=0)
        else:
            pass
            dtimes_delta = (dtimes[1:] - dtimes[:-1])
            names = meteva_base.get_stadata_names(sta1)
            sta1["weight_pre"] =0
            sta1["weight_later"] = 0
            for i in range(len(dtimes)-1):
                sta1.loc[sta1["dtime"] == dtimes[i+1],"weight_pre"] = dtimes_delta[i]
                sta1.loc[sta1["dtime"] == dtimes[i], "weight_later"] = dtimes_delta[i]

            sta2 = sta1.iloc[:,:-2]
            sta2.loc[:,"count_for_add"] = sta1.loc[:,"weight_pre"]
            rain_ac = sta2.copy()


            for n in range(len(names)):
                rain_ac.loc[:,names[n]] *= sta2.loc[:,"count_for_add"]

            sta2.loc[:,"count_for_add"] = sta1.loc[:,"weight_pre"] + sta1.loc[:,"weight_later"]
            for i in range(1,step-1):
                rain1 = sta2.copy()
                for n in range(len(names)):
                    rain1.loc[:,names[n]] *= sta2.loc[:,"count_for_add"]
                rain1["dtime"] = rain1["dtime"] + dhour_unit * i
                rain_ac = meteva_base.add_on_level_time_dtime_id(rain_ac, rain1,how="outer",default=0)

            sta2.loc[:,"count_for_add"] = sta1.loc[:,"weight_later"]
            rain1 = sta2.copy()
            for n in range(len(names)):
                rain1.loc[:,names[n]] *= sta2.loc[:,"count_for_add"]
            rain1["dtime"] = rain1["dtime"] + dhour_unit * step
            rain_ac = meteva_base.add_on_level_time_dtime_id(rain_ac, rain1, how="outer", default=0)

        names = meteva_base.get_stadata_names(rain_ac)
        for n in range(len(names)-1):
            rain_ac[names[n]] /= rain_ac[names[-1]]

        rain_ac = meteva_base.in_member_list(rain_ac,names[:-1])
        rain_ac = meteva_base.in_dtime_list(rain_ac,dtimes[1:])  # 删除时效小于range的部分
        if not keep_all:
            dh = ((dtimes - dtimes[-1]) / dhour_unit).astype(np.int32)
            new_dtimes = dtimes[dh % step == 0]
            rain_ac = meteva_base.in_dtime_list(rain_ac, new_dtimes)
        rain_ac.attrs["valid_time"] = span
        return rain_ac

def std_of_sta(sta,used_coords = ["member"]):
    sta_std = sta.loc[:,meteva_base.get_coord_names()]
    sta_data = sta[meteva_base.get_stadata_names(sta)]
    value = sta_data.values.astype(np.float32)
    std = np.std(value, axis=1)
    sta_std['std'] = std
    return sta_std

def var_of_sta(sta,used_coords = ["member"]):
    sta_var = sta.loc[:,meteva_base.get_coord_names()]
    sta_data = sta[meteva_base.get_stadata_names(sta)]
    value = sta_data.values
    var = np.var(value, axis=1)
    sta_var['var'] = var
    return sta_var

def max_of_sta(sta,used_coords = ["member"],span = None, contain_start = False,keep_all = True,ignore_missing = False):
    if not isinstance(used_coords,list):
        used_coords = [used_coords]

    default = np.min(sta.iloc[:,6:].values)

    if ignore_missing:
        how = "outer"
    else:
        how = "inner"

    if used_coords  == ["member"]:
        sta_max = sta.loc[:,meteva_base.get_coord_names()]
        sta_data = sta[meteva_base.get_stadata_names(sta)]
        value = sta_data.values
        max1 = np.max(value, axis=1)
        sta_max['max'] = max1
        return sta_max
    elif used_coords == ["time"]:
        if span is None:
            times = sta.loc[:, 'time'].values
            times = list(set(times))
            times.sort()
            rain_ac = meteva_base.in_time_list(sta,[times[0]])
            meteva_base.set_stadata_coords(rain_ac, time=times[-1])
            for i in range(1,len(times)):
                rain01 = meteva_base.in_time_list(sta,times[i])
                meteva_base.set_stadata_coords(rain01,time=times[-1])
                rain_ac = meteva_base.max_on_level_time_dtime_id(rain_ac, rain01, how=how,default= default)
            return rain_ac
        else:
            times = sta.loc[:, 'time'].values
            times = list(set(times))
            times.sort()
            times = np.array(times)
            dtimes = times[1:] - times[0:-1]
            min_dtime = np.min(dtimes)
            min_dhour = min_dtime / np.timedelta64(1, 'h')
            rain_ac = None
            step = int(round(span/min_dhour))
            if contain_start: step += 1
            for i in range(step):
                rain1 = sta.copy()
                rain1["time"] = rain1["time"] + min_dtime * i
                rain_ac = meteva_base.max_on_level_time_dtime_id(rain_ac, rain1, default=default)
            time0_add_span = times[0] + np.timedelta64(int(min_dhour * (step - 1)),'h')
            rain_ac = meteva_base.sele_by_para(rain_ac,time_range=[time0_add_span,times[-1]])
            if not keep_all:
                dtimes = times[:] - times[-1]
                dh = (dtimes / min_dtime).astype(np.int32)
                new_times = times[dh % step == 0]
                rain_ac = meteva_base.in_time_list(rain_ac, new_times)
            return rain_ac
    elif used_coords ==["dtime"]:
        if span is None:
            dtimes = sta.loc[:, 'dtime'].values
            dtimes = list(set(dtimes))
            dtimes.sort()
            rain_ac = meteva_base.in_dtime_list(sta, [dtimes[0]])
            meteva_base.set_stadata_coords(rain_ac, dtime=dtimes[-1])
            for i in range(1, len(dtimes)):
                rain01 = meteva_base.in_dtime_list(sta, dtimes[i])
                meteva_base.set_stadata_coords(rain01, dtime=dtimes[-1])
                rain_ac = meteva_base.max_on_level_time_dtime_id(rain_ac, rain01, how=how,default= default)
            return rain_ac
        else:
            dtimes = sta.loc[:, 'dtime'].values
            dtimes = list(set(dtimes))
            dtimes.sort()
            dtimes = np.array(dtimes)
            dhour_unit = dtimes[1] - dtimes[0]

            #if dhour_unit == 0:
            #    dhour_unit = dtimes[1]

            rain_ac = sta.copy()
            #print(span)
            #print(dhour_unit)
            step = int(round(span/dhour_unit))
            if contain_start: step += 1
            for i in range(1,step):
                rain1 = sta.copy()
                rain1["dtime"] = rain1["dtime"] + dhour_unit * i
                # print(dhour_unit * i)
                rain_ac = meteva_base.max_on_level_time_dtime_id(rain_ac, rain1, default=default)


            begin_dtime = dtimes[0]+dhour_unit * (step - 1)
            rain_ac = meteva_base.between_dtime_range(rain_ac,begin_dtime,dtimes[-1])  # 删除时效小于range的部分
            dtimes =np.array(list(set(rain_ac.loc[:, "dtime"].values.tolist())))
            if not keep_all:
                dh = ((dtimes - dtimes[-1]) / dhour_unit).astype(np.int32)
                new_dtimes = dtimes[dh % step == 0]
                rain_ac = meteva_base.in_dtime_list(rain_ac, new_dtimes)
            return rain_ac

def min_of_sta(sta,used_coords = ["member"],span = None,contain_start = False,keep_all = True,ignore_missing = False):
    if not isinstance(used_coords,list):
        used_coords = [used_coords]

    default = np.max(sta.iloc[:,6:].values)
    if ignore_missing:
        how = "outer"
    else:
        how = "inner"
    if used_coords  == ["member"]:
        sta_min = sta.loc[:,meteva_base.get_coord_names()]
        sta_data = sta[meteva_base.get_stadata_names(sta)]
        value = sta_data.values
        min1 = np.min(value, axis=1)
        sta_min['min'] = min1
        return sta_min
    elif used_coords == ["time"]:
        if span is None:
            times = sta.loc[:, 'time'].values
            times = list(set(times))
            times.sort()
            rain_ac = meteva_base.in_time_list(sta,[times[0]])
            meteva_base.set_stadata_coords(rain_ac, time=times[-1])
            for i in range(1,len(times)):
                rain01 = meteva_base.in_time_list(sta,times[i])
                meteva_base.set_stadata_coords(rain01,time=times[-1])
                rain_ac = meteva_base.min_on_level_time_dtime_id(rain_ac, rain01, how=how,default= default)
            return rain_ac
        else:
            times = sta.loc[:, 'time'].values
            times = list(set(times))
            times.sort()
            times = np.array(times)
            dtimes = times[1:] - times[0:-1]
            min_dtime = np.min(dtimes)
            min_dhour = min_dtime / np.timedelta64(1, 'h')
            rain_ac = None
            step = int(round(span/min_dhour))
            if contain_start:step += 1
            for i in range(step):
                rain1 = sta.copy()
                rain1["time"] = rain1["time"] + min_dtime * i
                rain_ac = meteva_base.min_on_level_time_dtime_id(rain_ac, rain1, default= default)
            time0_add_span = times[0] + np.timedelta64(int(min_dhour * (step - 1)),'h')
            rain_ac = meteva_base.sele_by_para(rain_ac,time_range=[time0_add_span,times[-1]])
            if not keep_all:
                dtimes = times[:] - times[-1]
                dh = (dtimes / min_dtime).astype(np.int32)
                new_times = times[dh % step == 0]
                rain_ac = meteva_base.in_time_list(rain_ac, new_times)
            return rain_ac
    elif used_coords ==["dtime"]:
        if span is None:
            dtimes = sta.loc[:, 'dtime'].values
            dtimes = list(set(dtimes))
            dtimes.sort()
            rain_ac = meteva_base.in_dtime_list(sta, [dtimes[0]])
            meteva_base.set_stadata_coords(rain_ac, dtime=dtimes[-1])
            for i in range(1, len(dtimes)):
                rain01 = meteva_base.in_dtime_list(sta, dtimes[i])
                meteva_base.set_stadata_coords(rain01, dtime=dtimes[-1])
                rain_ac = meteva_base.min_on_level_time_dtime_id(rain_ac, rain01, how=how,default= default)
            return rain_ac
        else:
            dtimes = sta.loc[:, 'dtime'].values
            dtimes = list(set(dtimes))
            dtimes.sort()
            dtimes = np.array(dtimes)
            dhour_unit = dtimes[1] - dtimes[0]

            #if dhour_unit == 0:
            #    dhour_unit = dtimes[1]

            rain_ac = sta.copy()
            #print(span)
            #print(dhour_unit)
            step = int(round(span/dhour_unit))
            if contain_start:step += 1

            for i in range(1,step):
                rain1 = sta.copy()
                rain1["dtime"] = rain1["dtime"] + dhour_unit * i
                # print(dhour_unit * i)
                rain_ac = meteva_base.min_on_level_time_dtime_id(rain_ac, rain1,default=default)

            begin_dtime = dtimes[0]+dhour_unit * (step - 1)
            rain_ac = meteva_base.between_dtime_range(rain_ac,begin_dtime,dtimes[-1])  # 删除时效小于range的部分
            dtimes =np.array(list(set(rain_ac.loc[:, "dtime"].values.tolist())))
            if not keep_all:
                dh = ((dtimes - dtimes[-1]) / dhour_unit).astype(np.int32)
                new_dtimes = dtimes[dh % step == 0]
                rain_ac = meteva_base.in_dtime_list(rain_ac, new_dtimes)
            return rain_ac

def sum_of_sta(sta,used_coords = ["member"],span = None,keep_all = True):
    if not isinstance(used_coords,list):
        used_coords = [used_coords]

    if used_coords == ["member"]:
        sta_sum = sta.loc[:,meteva_base.get_coord_names()]
        sta_data = sta[meteva_base.get_stadata_names(sta)]
        value = sta_data.values
        min1 = np.sum(value, axis=1)
        sta_sum['sum'] = min1
        return sta_sum

    elif used_coords == ["time"]:
        if span is None:
            times = sta.loc[:, 'time'].values
            times = list(set(times))
            times.sort()
            rain_ac = meteva_base.in_time_list(sta,[times[0]])
            meteva_base.set_stadata_coords(rain_ac, time=times[-1])
            for i in range(1,len(times)):
                rain01 = meteva_base.in_time_list(sta,times[i])
                meteva_base.set_stadata_coords(rain01,time=times[-1])
                rain_ac = meteva_base.add_on_level_time_dtime_id(rain_ac, rain01, how="outer",default=0)
            return rain_ac
        else:
            times = sta.loc[:, 'time'].values
            times = list(set(times))
            times.sort()
            times = np.array(times)
            dtimes = times[1:] - times[0:-1]
            min_dtime = np.min(dtimes)
            min_dhour = min_dtime / np.timedelta64(1, 'h')
            rain_ac = None
            step = int(round(span/min_dhour))
            for i in range(step):
                rain1 = sta.copy()
                rain1["time"] = rain1["time"] + min_dtime * i
                rain_ac = meteva_base.add_on_level_time_dtime_id(rain_ac, rain1, how="inner")

            time0_add_span = times[0] + np.timedelta64(int(min_dhour * (step - 1)),'h')
            rain_ac = meteva_base.sele_by_para(rain_ac,time_range=[time0_add_span,times[-1]])
            if not keep_all:
                dtimes = times[:] - times[-1]
                dh = (dtimes / min_dtime).astype(np.int32)
                new_times = times[dh % step == 0]
                rain_ac = meteva_base.in_time_list(rain_ac, new_times)
            return rain_ac
    elif used_coords ==["dtime"]:
        if span is None:
            dtimes = sta.loc[:, 'dtime'].values
            dtimes = list(set(dtimes))
            dtimes.sort()
            rain_ac = meteva_base.in_dtime_list(sta, [dtimes[0]])
            meteva_base.set_stadata_coords(rain_ac, dtime=dtimes[-1])
            for i in range(1, len(dtimes)):
                rain01 = meteva_base.in_dtime_list(sta, dtimes[i])
                meteva_base.set_stadata_coords(rain01, dtime=dtimes[-1])
                rain_ac = meteva_base.add_on_level_time_dtime_id(rain_ac, rain01, how="outer",default=0)

                # rain_ac0 = rain_ac.copy()
                # a = rain_ac0.duplicated(subset=["level","time","dtime","id"])
                # for ii in range(len(a)):
                #     if a[ii]:
                #         ids = rain_ac["id"].values[ii]
                #         sta4 = meteva_base.sele_by_para(rain_ac,id = ids)
                #         print(sta4)
                #         print()

                #rain_ac.drop_duplicates(subset=["level","time","dtime","id"],inplace=True)
            return rain_ac
        else:
            dtimes = sta.loc[:, 'dtime'].values
            dtimes = list(set(dtimes))
            dtimes.sort()
            dtimes = np.array(dtimes)
            dhour_unit = dtimes[1] - dtimes[0]

            #if dhour_unit == 0:
            #    dhour_unit = dtimes[1]

            rain_ac = sta.copy()
            #print(span)
            #print(dhour_unit)
            step = int(round(span/dhour_unit))
            #print(step)
            for i in range(1,step):
                rain1 = sta.copy()
                rain1["dtime"] = rain1["dtime"] + dhour_unit * i
                # print(dhour_unit * i)
                rain_ac = meteva_base.add_on_level_time_dtime_id(rain_ac, rain1, default=0)

            begin_dtime = dtimes[0]+dhour_unit * (step - 1)
            rain_ac = meteva_base.between_dtime_range(rain_ac,begin_dtime,dtimes[-1])  # 删除时效小于range的部分
            dtimes =set(rain_ac.loc[:, "dtime"].values.tolist())
            dtimes = list(dtimes)
            dtimes.sort()
            dtimes = np.array(dtimes)
            if not keep_all:
                dh = ((dtimes - dtimes[-1]) / dhour_unit).astype(np.int32)
                new_dtimes = dtimes[dh % step == 0]
                rain_ac = meteva_base.in_dtime_list(rain_ac, new_dtimes)
            return rain_ac

#获取网格数据的平均值
def mean_of_grd(grd,used_coords = ["member"],span = None):

    if used_coords==["member"]:
        grid0 = meteva_base.basicdata.get_grid_of_data(grd)
        grid1 = meteva_base.basicdata.grid(grid0.glon,grid0.glat,grid0.gtime,grid0.dtimes,grid0.levels,member_list=["mean"])
        dat = np.squeeze(grd.values)
        if len(dat.shape) > 2:
            dat = np.mean(dat,axis = 0)
        grd1 = meteva_base.basicdata.grid_data(grid1,dat)
        return grd1
    elif used_coords == "dtime" or used_coords == ["dtime"]:
        if span == None:
            grid0 = meteva_base.get_grid_of_data(grd)
            dtimes = np.array(grid0.dtimes)
            dtimes_sum = [dtimes[-1]]
            grid1 = meteva_base.grid(grid0.glon, grid0.glat, grid0.gtime, dtimes_sum, grid0.levels, grid0.members)
            grd_mean = meteva_base.grid_data(grid1)
            grd_mean.values[:, :, :, 0, :, :] = np.mean(grd.values[:, :, :, :, :, :], axis=3)
            return grd_mean
        else:
            grid0 = meteva_base.get_grid_of_data(grd)
            dtimes = np.array(grid0.dtimes)
            dtimes_sum = dtimes[dtimes >= span]
            grid1 = meteva_base.grid(grid0.glon, grid0.glat, grid0.gtime, dtimes_sum, grid0.levels, grid0.members)
            grd_mean = meteva_base.grid_data(grid1)
            for i in range(len(dtimes_sum)):
                dtime_e = dtimes_sum[i]
                dtime_s = dtimes_sum[i] - span
                index = np.where((dtimes > dtime_s) & (dtimes <= dtime_e))[0]

                grd_mean.values[:, :, :, i, :, :] = np.mean(grd.values[:, :, :, index, :, :], axis=3)
            return grd_mean
    elif used_coords =="time" or used_coords ==["time"]:
        if span ==None:
            grid0 = meteva_base.get_grid_of_data(grd)
            grid1 = meteva_base.grid(grid0.glon, grid0.glat, [grid0.gtime[1]], grid0.dtimes, grid0.levels,
                                     grid0.members)
            grd_mean = meteva_base.grid_data(grid1)

            grd_mean.values[:, :, 0, :, :, :] = np.mean(grd.values[:, :, :, :, :, :], axis=2)
            return grd_mean
        else:
            grid0 = meteva_base.get_grid_of_data(grd)
            grd_mean = meteva_base.grid_data(grid0)
            values = grd.values.copy()
            step = grid0.dtime_int
            half = int(span/step/2)
            kernal = np.ones(half * 2 +1)
            kernal[0] = 0.5
            kernal[-1] = 0.5
            shape = values.shape
            ntime = values.shape[2]
            for i in range(ntime):
                sum_v = np.zeros((shape[0],shape[1],shape[3],shape[4],shape[5]))
                sum_q = 0
                for k in range(-half,half+1,1):
                    i_k = i + k
                    if i_k>=0 and i_k< values.shape[2]:
                        q = kernal[k + half]
                        sum_v += grd.values[:, :, i_k, :, :, :] * q
                        sum_q+= q
                sum_v /= sum_q
                grd_mean.values[:, :, i, :, :, :] = sum_v

            return grd_mean



#获取网格数据的方差
def var_of_grd(grd,used_coords = ["member"]):
    grid0 = meteva_base.basicdata.get_grid_of_data(grd)
    grid1 = meteva_base.basicdata.grid(grid0.glon,grid0.glat,grid0.gtime,grid0.dtimes,grid0.levels,member_list=["var"])
    dat = np.squeeze(grd.values)
    if len(dat.shape) > 2:
        dat = np.var(dat,axis = 0)
    grd1 = meteva_base.basicdata.grid_data(grid1,dat)
    return grd1

#获取网格数据的标准差
def std_of_grd(grd,used_coords = ["member"]):
    grid0 = meteva_base.basicdata.get_grid_of_data(grd)
    grid1 = meteva_base.basicdata.grid(grid0.glon,grid0.glat,grid0.gtime,grid0.dtimes,grid0.levels,member_list=["std"])
    dat = np.squeeze(grd.values)
    if len(dat.shape) > 2:
        dat = np.std(dat,axis = 0)
    grd1 = meteva_base.basicdata.grid_data(grid1,dat)
    return grd1

#获取网格数据的最小值
def min_of_grd(grd,used_coords = ["member"]):
    grid0 = meteva_base.basicdata.get_grid_of_data(grd)
    grid1 = meteva_base.basicdata.grid(grid0.glon,grid0.glat,grid0.gtime,grid0.dtimes,grid0.levels,member_list=["min"])
    dat = np.squeeze(grd.values)
    if len(dat.shape)>2:
        dat = np.min(dat,axis = 0)
    grd1 = meteva_base.basicdata.grid_data(grid1,dat)
    return grd1

#获取网格数据的最大值
def max_of_grd(grd,used_coords = ["member"]):
    grid0 = meteva_base.basicdata.get_grid_of_data(grd)
    grid1 = meteva_base.basicdata.grid(grid0.glon,grid0.glat,grid0.gtime,grid0.dtimes,grid0.levels,member_list=["max"])
    dat = np.squeeze(grd.values)
    if len(dat.shape)>2:
        dat = np.max(dat,axis = 0)
    grd1 = meteva_base.basicdata.grid_data(grid1,dat)
    return grd1

#获取网格数据的求和
def sum_of_grd(grd,used_coords = ["member"],span = None, keep_all=True):
    if used_coords ==["member"]:
        grid0 = meteva_base.basicdata.get_grid_of_data(grd)
        grid1 = meteva_base.basicdata.grid(grid0.glon,grid0.glat,grid0.gtime,grid0.dtimes,grid0.levels,member_list=["max"])
        dat = np.squeeze(grd.values)
        if len(dat.shape) > 2:
            dat = np.sum(dat,axis = 0)
        grd1 = meteva_base.basicdata.grid_data(grid1,dat)
        return grd1
    elif used_coords == "dtime" or used_coords ==["dtime"]:
        if span == None:
            grid0 = meteva_base.get_grid_of_data(grd)
            dtimes = np.array(grid0.dtimes)
            dtimes_sum = [dtimes[-1]]
            grid1 = meteva_base.grid(grid0.glon, grid0.glat, grid0.gtime, dtimes_sum, grid0.levels, grid0.members)
            grd_sum = meteva_base.grid_data(grid1)
            grd_sum.values[:, :, :, 0, :, :] = np.sum(grd.values[:, :, :, :, :, :], axis=3)
        else:
            grid0 = meteva_base.get_grid_of_data(grd)
            dtimes = np.array(grid0.dtimes)
            dtimes_sum = dtimes[dtimes>=span]
            grid1 = meteva_base.grid(grid0.glon,grid0.glat,grid0.gtime,dtimes_sum,grid0.levels,grid0.members)
            grd_sum = meteva_base.grid_data(grid1)
            for i in range(len(dtimes_sum)):
                dtime_e = dtimes_sum[i]
                dtime_s = dtimes_sum[i] - span
                index = np.where((dtimes > dtime_s) & (dtimes <= dtime_e))[0]

                grd_sum.values[:,:,:,i,:,:] = np.sum(grd.values[:,:,:,index,:,:],axis=3)
        if not keep_all: #只保留时刻预报时效
            fhs = np.sort(grd_sum.dtime.values)
            fhs0 = np.arange(fhs[-1], fhs[0]-1, 0-span)[::-1]
            grd_sum = grd_sum.sel(dtime = fhs0.tolist())
        return grd_sum
    elif used_coords =="time" or used_coords ==["time"]:
        if span ==None:
            grid0 = meteva_base.get_grid_of_data(grd)
            grid1 = meteva_base.grid(grid0.glon, grid0.glat, [grid0.gtime[1]], grid0.dtimes, grid0.levels, grid0.members)
            grd_sum = meteva_base.grid_data(grid1)
            grd_sum.values[:, :, 0, :, :, :] = np.sum(grd.values[:, :, :, :, :, :], axis=2)
        else:
            grid0 = meteva_base.get_grid_of_data(grd)
            grd_sum = meteva_base.grid_data(grid0)
            values = grd.values.copy()
            step = grid0.dtime_int
            count = int(span/step)

            shape = values.shape
            ntime = values.shape[2]
            for i in range(ntime):
                sum_v = np.zeros((shape[0],shape[1],shape[3],shape[4],shape[5]))
                for k in range(-count+1,1):
                    i_k = i + k
                    if i_k <0:continue
                    sum_v += grd.values[:, :, i_k, :, :, :]
                grd_sum.values[:, :, i, :, :, :] = sum_v

            grd_sum = grd_sum.isel(time = slice(count-1,ntime))
        return grd_sum



def time_ceilling(sta,step = 1, time_unit = "h",begin_hour= 8):
    '''
    将不规则时间观测数据累计到固定步长的整点时刻
    :param sta: 站点数据，例如原始的闪电观测数据
    :param step:  累计的步长
    :param time_unit: 累计的时间单位，可选项包括 “H"和”M"，分别代表小时和分钟。
    :param begin_hour: 时间类型，当累计步长超过1小时,例如3小时，起步累计的时间是从08时还是00时，通常对北京时数据来说以08时起步，世界时则以00时起步
    :return:
    '''
    sta1 = sta.copy()
    time0 = datetime.datetime(2000,1,1,begin_hour,0)
    if time_unit.lower() == "h":
        step *= 3600
    else:
        step *= 60
    delta = ((sta["time"] - time0)/np.timedelta64(1,"s")).values
    delta =np.ceil(delta/step) * step
    sta1["time"] = time0
    sta1["time"] += delta * np.timedelta64(1, "s")
    return sta1

def loc_of_max(sta,used_coords = ["dtime"],ignore_missing = False):
    '''
    返回站点数据在某些维度上最大值的坐标
    :param sta:
    :param used_coords:
    :param span:
    :param keep_all:
    :param ignore_missing:
    :return:
    '''
    if not isinstance(used_coords,list):
        used_coords = [used_coords]

    default = np.min(sta.iloc[:,6:].values)
    if ignore_missing:
        how = "outer"
    else:
        how = "inner"
    if used_coords  == ["dtime"]:
        dtimes = sta.loc[:, 'dtime'].values
        dtimes = list(set(dtimes))
        dtimes.sort()
        dtime_array = np.array(dtimes)
        data_names = meteva_base.get_stadata_names(sta)
        max_loc = None
        for data_name in data_names:
            sta1 = meteva_base.in_member_list(sta,[data_name])

            sta1_0 = meteva_base.in_dtime_list(sta1,[dtimes[0]])
            meteva_base.set_stadata_coords(sta1_0, dtime=0)
            for i in range(1, len(dtimes)):
                sta1_1 = meteva_base.in_dtime_list(sta1, dtimes[i])
                meteva_base.set_stadata_coords(sta1_1, dtime=0)
                meteva_base.set_stadata_names(sta1_1,data_name_list=[data_name+"_"+str(i)])
                sta1_0 = pd.merge(sta1_0, sta1_1, on=["level", "time", "dtime", "id","lon","lat"], how=how)

            sta1_0.fillna(default, inplace=True)

            all_values = sta1_0.iloc[:, 6:].values
            max_index = np.argmax(all_values,axis=1)
            max_dtime = dtime_array[max_index]
            sta1_dtime = meteva_base.in_member_list(sta1_0,[data_name])
            sta1_dtime.iloc[:,-1] = max_dtime[:]
            max_loc = meteva_base.combine_on_level_time_dtime_id(max_loc,sta1_dtime)

        return max_loc

def loc_of_min(sta,used_coords = ["dtime"],ignore_missing = False):
    sta1 = sta.copy()
    sta1.iloc[:,6:] *= -1
    return loc_of_max(sta1,used_coords=used_coords,ignore_missing = ignore_missing)



######### 格点数据时间拆分技术  #########

def read_multi_griddata(fmt, time, fhs, io_method=None, is_ob=False, is_fill0=True):
    """
    读取多时次网格数据，并merge融合为DataSet输出
    is_ob: 是否实况。是：time=time+fh,fh=0.
    is_fill0: 当dtime=0时，是否自动赋值value=0
    """
    import xarray as xr
    if io_method == None:
        io_method = meb.read_griddata_from_nc
    grd_list =[]
    flag0 = 0 ##是否需要补fh=0时刻的0值
    for i,fh in enumerate(fhs):
        if is_ob:
            time0 = time+datetime.timedelta(hours=int(fh))
            fh0 = 0
        else:
            time0 = time
            fh0 = fh
        tp03_file = meb.get_path(fmt, time=time0,  dt=int(fh0))
        if i%5==0: print('Read FILE: ',i, time0, fh0, tp03_file)
        if not os.path.exists(tp03_file):
            print('File missing: {}-{}'.format(time0, fh0))
            continue
        if not is_fill0:#不需要补0，直接读取fh=0
            grd = io_method(tp03_file)
            meb.set_griddata_coords(grd, gtime=[time0], dtime_list=[fh0])
            grd_list.append(grd)
        else:#无fh=0的数据，直接补0
            if fh0==0:#为0时，需要补充
                flag0 = 1
            else:
                grd = io_method(tp03_file)
                meb.set_griddata_coords(grd, gtime=[time0], dtime_list=[fh0])
                if flag0:#先补充fh=0的零值网格数据
                    grd_00 = grd.copy()
                    grd_00.values = np.zeros_like(grd_00.values)
                    meb.set_griddata_coords(grd_00, dtime_list=[0])
                    grd_list.append(grd_00)
                    flag0 = 0
                grd_list.append(grd)
    ## 组合数据
    try:
        grd_list0 = [i.astype(np.float32) for i in grd_list]
        del(grd_list)
        grd_all = meb.concat(grd_list0)
    except Exception as err:
        print(err)
        return None
    return grd_all


def cal_griddata_TimeSplit(input_fmt,  time, output_fmt=None,
    input_fh_list=np.arange(3,73,3), output_fh_list=np.arange(0,73,1),  interp_method='slinear', io_method = None,
    uv_fmt = None, #如非None,风速合成
    delta_hour = None, #delta变量
    is_fill0 = False, #fh=0时是读取文件或填0值
    ):
    """
    时间拆分预处理，包括uv生成风速、逐delta小时相减等计算
    """
    ## read data
    fh03_list = input_fh_list
    fh01_list = output_fh_list
    try:
        ## 组合数据 
        grd_all = read_multi_griddata(input_fmt, time, fh03_list, io_method=io_method, is_fill0=is_fill0)#DataArray
        print(grd_all)
        ## 预处理, UV风读取并合成风速
        if uv_fmt is not None:
            grd_0 = read_multi_griddata(uv_fmt, time, fh03_list, io_method=io_method, is_fill0=is_fill0)#读取v分量
            print(grd_0)
            # grd_all, _ = cal_wswd_from_meteva_uvgrid(grd_all, grd_0)#计算风速
            grd_all, _ = meb.u_v_to_speed_angle(grd_all, grd_0)#计算风速
        ## 拆分数据
        grd_all_01h = grd_all.interp(dtime=fh01_list, method=interp_method, kwargs={"fill_value": "extrapolate"})    
        ## 后处理，计算时间变量(delta)
        if delta_hour is not None:
            grd_all_01h = meb.change(grd_all_01h, delta=delta_hour, used_coords="dtime")
    except Exception as err:
        print(err)
        return None,None
    # 输出
    if output_fmt is not None:#输出拆分结果nc
        for fh01 in fh01_list:
            out_file = meb.get_path(output_fmt, time=time, dt=fh01)
            if not os.path.exists(out_file):
                try:
                    grd_01h = grd_all_01h.sel(dtime=[fh01]).astype(np.float32)
                    # grd_01h = grd_01h.astype(np.float32)
                    grd_01h.values[grd_01h.values<0] = 0#最小为0
                    meb.write_griddata_to_nc(grd_01h, out_file, effectiveNum=2, creat_dir=True)
                    print(out_file)
                except Exception as err:
                    print(err, time, fh01)
                    continue
    return(grd_all_01h, grd_all)


class EnsData_Pre_rain03(object):
    """
    集合预报(网格数据)预处理，多时次预报合并，拆分至03h，从tp中计算rain03(时段累积)
    """
    def __init__(self,  input_fmt=None, output_fmt=None, input_fh_list=np.arange(0,73,3), output_fh_list=np.arange(0,73,1),
                        delta_hour=None,  interp_method='slinear', io_method = None):
        self.input_fmt      = input_fmt
        self.output_fmt     = output_fmt 
        self.input_fh_list  = input_fh_list
        self.output_fh_list = output_fh_list
        self.delta_hour     = delta_hour
        self.method         = interp_method
        self.io_method      = io_method

    def process(self, times):
        for time in times:
            print('Calculation: {}'.format(time))
            grd_03, grd_all = cal_griddata_TimeSplit(input_fmt=self.input_fmt, time=time, output_fmt=self.output_fmt,
                        input_fh_list=self.input_fh_list, output_fh_list=self.output_fh_list,
                        delta_hour=self.delta_hour,
                        interp_method=self.method, io_method=self.io_method, is_fill0=True)
        return grd_03, grd_all

######### 格点数据时间累加技术  #########
def sum_grd01h_to_03_24h(input_fmt, time, output_fmt=None, output_fmt2=None,
    input_fh_list=np.arange(1,25,1), 
    span1=3, span2=None, 
    io_method=None, is_ob=True):
    """
    将逐小时网格数据累加为其他时效(逐3h或逐24h<可选>)
    is_ob: True-处理为观测时间time，fh=0； False-与预报一直的time+fh
    """
    import xarray as xr
    fh01_list = input_fh_list
    grd_all = read_multi_griddata(input_fmt, time, fh01_list, io_method=io_method, is_ob=is_ob)#DataArray
    grd0 = grd_all
    try:
        ## 累加并提取
        grd03 = meb.sum_of_grd(grd0, used_coords="time", span=span1)#包括全部03h
        grd03 = _sel_keep_True(grd03, time, time0, add_hour=0-span1)
        if span2 is not None:
            grd24 = meb.sum_of_grd(grd0, used_coords="time", span=span2)#包括全部24h
            grd24 = _sel_keep_True(grd24, time, time0, add_hour=0-span2)
        else:
            grd24 = None
        # 输出
        if output_fmt is not None:#输出拆分结果nc
            fh03_list = grd03.dtime.values
            for fh03 in fh03_list:
                out_file = meb.get_path(output_fmt, time=time, dt=fh03)
                if not os.path.exists(out_file):
                    try:
                        grd = grd03.sel(dtime=[fh03])
                        grd.values[grd.values<0] = 0#最小为0
                        meb.write_griddata_to_nc(grd, out_file, effectiveNum=2, creat_dir=True)
                    except Exception as err:
                        print(err, time, fh03)
                        continue
        if span2 is not None and output_fmt2 is not None :#输出拆分结果nc
            fh24_list = grd24.dtime.values
            for fh24 in fh24_list:
                out_file = meb.get_path(output_fmt2, time=time, dt=fh24)
                if not os.path.exists(out_file):
                    try:
                        grd = grd24.sel(dtime=[fh24])
                        grd.values[grd.values<0] = 0#最小为0
                        meb.write_griddata_to_nc(grd, out_file, effectiveNum=2, creat_dir=True)
                    except Exception as err:
                        print(err, time, fh24)
                        continue
        return(grd03, grd24)
    except Exception as err:
        print(Exception)
        return None,None
        

def _sel_keep_True(grd, time, time0, add_hour=-3):
    ## 3h累加及挑选
    times03 = mtl.get_date_list_pd([time, time0, add_hour])
    times03.sort()
    dim_time = grd.coords["time"]#time维度
    try:
        temp = np.array([meb.all_type_time_to_datetime(i) for i in dim_time.values])
        flag = np.isin(temp, times03)  #挑选合适的flag(time维度isin对应的times03范围内)
        grd = grd.loc[
            dict(time=flag)
        ]
        return grd
    except Exception as err:
        print(err)
        return None


#!/usr/bin/python3.6
# -*- coding:UTF-8 -*-
import meteva
from scipy.spatial import cKDTree
import numpy as np

ER = 6371
def get_nearby_sta_index_ensemble(sta, grid,nearNum = 1):
    members = np.arange(nearNum).tolist()
    grid1 = meteva.base.grid(grid.glon,grid.glat,members=members)
    grd_en = meteva.base.grid_data(grid1)
    xyz_sta =  meteva.base.tool.math_tools.lon_lat_to_cartesian(sta.loc[:,"lon"], sta.loc[:,"lat"],R = ER)
    lon = np.arange(grid1.nlon) * grid1.dlon + grid1.slon
    lat = np.arange(grid1.nlat) * grid1.dlat + grid1.slat
    grid_lon,grid_lat = np.meshgrid(lon,lat)
    xyz_grid = meteva.base.tool.math_tools.lon_lat_to_cartesian(grid_lon.flatten(), grid_lat.flatten(),R = ER)
    tree = cKDTree(xyz_sta)
    value, inds = tree.query(xyz_grid, k=nearNum)
    grd_en.values = inds.reshape((nearNum,1,1,1,grid1.nlat,grid1.nlon))
    return grd_en



import numpy as np
import math
import meteva
from scipy.spatial import cKDTree
import time

def transform(sta,dlon = None,dlat = None):
    """
    将站点形式的规则网格的数据转化为格点数据
    :param sta:站点数据
    :param dlon 经度精度
    :param dlat 纬度经度
    :return:返回格点网格数据
    """
    slon = np.min(sta['lon'])
    elon = np.max(sta['lon'])
    slat = np.min(sta['lat'])
    elat = np.max(sta['lat'])
    nsta = len(sta.index)
    if(dlon is None):
        for i in range(nsta-1):
            dlon = sta['lon'].values[i] - sta['lon'].values[i+1]
            if dlon != 0:
                dlon = math.fabs(dlon)
                break
    if(dlat is None):
        for i in range(nsta-1):
            dlat = sta['lat'].values[i] - sta['lat'].values[i+1]
            if dlat != 0:
                dlat = math.fabs(dlat)
                break
    ig = ((sta['lon'].values - slon) // dlon).astype(dtype = 'int16')
    jg = ((sta['lat'].values - slat) // dlat).astype(dtype = 'int16')
    grid0 = meteva.base.basicdata.grid([slon,elon,dlon],[slat,elat,dlat])
    dat = np.zeros((grid0.nlat,grid0.nlon))
    data_name = meteva.base.basicdata.get_data_names(sta)[0]
    dat[jg,ig] = sta.loc[:,data_name]
    grd = meteva.base.basicdata.grid_data(grid0,dat)
    return grd

#站点到格点的反距离插值，对每个格点，获取其最近的几个站点编号、距离, 然后计算权重和。
def sta_to_grid_idw(sta, grid0,background = None,effectR = 1000,nearNum = 8,other_info='left'):
    data_name = meteva.base.basicdata.get_data_names(sta)
    index0 = sta.index[0]
    if other_info=='left':
        grid = meteva.base.basicdata.grid(grid0.glon,grid0.glat,[sta.loc[index0,'time']],[sta.loc[index0,'dtime'],"h"],[sta.loc[index0,'level']],data_name)
    else:
        grid = grid0
    xyz_sta =  meteva.base.tool.math_tools.lon_lat_to_cartesian(sta['lon'].values, sta['lat'].values, R = meteva.base.basicdata.const.ER)
    lon = np.arange(grid.nlon) * grid.dlon + grid.slon
    lat = np.arange(grid.nlat) * grid.dlat + grid.slat
    grid_lon,grid_lat = np.meshgrid(lon,lat)
    xyz_grid = meteva.base.tool.math_tools.lon_lat_to_cartesian(grid_lon.flatten(), grid_lat.flatten(), R = meteva.base.basicdata.const.ER)
    tree = cKDTree(xyz_sta)
    #d,inds 分别是站点到格点的距离和id
    d, inds = tree.query(xyz_grid, k=nearNum)
    start = time.time()
    d += 1e-6
    w = 1.0 / d ** 2
    input_dat = sta['data0'].values
    dat = np.sum(w * input_dat[inds], axis=1) / np.sum(w, axis=1)
    end = time.time()
    print(end- start)
    bg = meteva.base.basicdata.grid_data(grid)
    if(background is not None):
        bg = meteva.base.function.gxy_gxy.interpolation_linear(background,grid)
    bg_dat = bg.values.flatten()
    dat = np.where(d[:,0] > effectR,bg_dat,dat)
    grd = meteva.base.basicdata.grid_data(grid,dat)
    grd.name = "data0"
    return grd

#站点到格点转换
def sta_to_grid_oa2(sta0,background,sm = 1,effect_R = 1000,rate_of_model = 0):

    sta = meteva.base.function.sxy_sxy.drop_nan(sta0)
    data_name = meteva.base.basicdata.get_data_names(sta)[0]
    grid = meteva.base.basicdata.get_grid_of_data(background)
    sta = meteva.base.function.get_from_sta_data.sta_in_grid_xy(sta, grid)
    #print(sta)
    grd = background.copy()
    dat = np.squeeze(grd.values)


    ig = ((sta['lon'].values - grid.slon) // grid.dlon).astype(dtype = 'int16')

    jg = ((sta['lat'].values - grid.slat) // grid.dlat).astype(dtype = 'int16')

    dx = (sta['lon'].values - grid.slon) / grid.dlon - ig

    dy = (sta['lat'].values - grid.slat) / grid.dlat - jg

    c00 = (1 - dx) * (1 - dy)

    c01 = dx * (1 - dy)

    c10 = (1-dx) * dy

    c11 = dx * dy

    lat = np.arange(grid.nlat) * grid.dlat + grid.slat

    sr = 1/np.power(np.cos(lat*math.pi/180),4)

    def targe(x):

        grdv =  x.reshape(grid.nlat,grid.nlon)

        dx = grdv[:,:-2] + grdv[:,2:] - 2 * grdv[:,1:-1]

        cost1 = np.sum(dx * dx)

        dy = grdv[:-2,:] + grdv[2:,:] - 2 * grdv[1:-1,:]

        dy2 = dy * dy

        sum_dy = np.sum(dy2,axis=1)

        cost1 = cost1 + np.dot(sum_dy,sr[1:-1])



        sta_g = c00 * dat[jg, ig] + c01 * dat[jg, ig + 1] + c10 * dat[jg + 1, ig] + c11 * dat[

            jg + 1, ig + 1]

        error = sta.ix[:,7] - sta_g

        cost2 = np.sum(error * error)

        cost = sm * cost1 + cost2

        return cost



    def grads(x):

        grdv = x.reshape(grid.nlat,grid.nlon)

        g1 = np.zeros(grdv.shape)

        dx = 2 * (grdv[:,:-2] + grdv[:,2:] - 2 * grdv[:,1:-1])

        g1[:,:-2] = dx

        g1[:,2:] += dx

        g1[:,1:-1] -= 2*dx



        sr_expend = np.tile(sr[1:-1],[grid.nlon,1]).T

        dy = 2 *(grdv[:-2,:] + grdv[2:,:] - 2 * grdv[1:-1,:])

        dy_sr = dy * sr_expend

        g1[:-2,:] += dy_sr

        g1[2:,:] += dy_sr

        g1[1:-1,:] -= 2 * dy_sr



        g2 = np.zeros(grdv.shape)

        sta_g = c00 * dat[jg, ig] + c01 * dat[jg, ig + 1] + c10 * dat[jg + 1, ig] + c11 * dat[jg + 1, ig + 1]

        d = 2 * (sta_g - sta.ix[:,7])

        g2[jg,ig] += d * c00

        g2[jg,ig + 1]  += d * c01

        g2[jg+1,ig] += d * c10

        g2[jg+1,ig+1] += d * c11

        g = sm * g1 + g2

        return g.reshape(-1)



    x = grd.values.reshape(-1)

    x_oa = meteva.base.method.frprmn2(x, targe, grads)

    grd.values = x_oa.reshape(1,1,1,1,grid.nlat,grid.nlon)

    return grd



import pandas as pd
import numpy as np
import copy
from scipy.spatial import cKDTree
import meteva


#将两个站点数据信息进行合并，并去重。
def set_data_to(sta,station):
    #删除重复行
    sta1 = sta.drop_duplicates(['id'])
    #先将数据合并
    df = pd.merge(station,sta1, on='id', how='left')
    #时间，时效和层次，采用sta的
    df.iloc[:, 0] = sta1.iloc[0, 0]
    df.iloc[:, 1] = sta1.iloc[0, 1]
    df.iloc[:, 2] = sta1.iloc[0, 2]

    #如果合并后sta对应的数据是缺省，则用station里的data0列补充
    columns = list(sta1.columns)
    len_c1 = len(columns)
    columns_m = list(df.columns)
    len_m = len(columns_m)
    for i in range(len_c1+6,len_m):
        name1 = columns_m[i]
        name2 = columns_m[7]
        df.loc[df[name1].isnull(), name1] = df[df[name1].isnull()][name2]


    #删除合并后多余的时空信息列
    len_s = len(list(station.columns))
    drop_col = list(df.columns[7:len_s+6])
    df.drop(drop_col, axis=1, inplace=True)
    #重新命名列名称
    df.columns = sta1.columns

    return df

#给站点信息的dataframe中的列名重新设置默认值
def set_default_value(sta,default):
    columns = list(sta.columns)
    len_c = len(columns)
    for i in range(7,len_c):
        name1 = columns[i]
        sta.loc[sta[name1].isnull(), name1] = default

#删除dataframe的nan值
def drop_nan(sta):
    columns = list(sta.columns)
    columns_data = columns[7:]
    df = sta.dropna(subset = columns_data)
    return df

#将两个站点dataframe相加在一起
def add_on_id(sta1_0, sta2_0, how="left", default=None):
    if sta1_0 is None:
        return sta2_0
    elif sta2_0 is None:
        return sta1_0
    else:
        # 删除重复行
        sta2 = sta2_0.drop_duplicates(['id'])
        sta1 = sta1_0.drop_duplicates(['id'])

        df = pd.merge(sta1, sta2, on='id', how=how)
        #print(len(sta1.index))
        #print(len(sta2.index))
        #print(len(df.index))
        #时间，时效和层次，采用df1的
        df.iloc[:, 0] = df.iloc[0, 0]
        df.iloc[:, 1] = df.iloc[0, 1]
        df.iloc[:, 2] = df.iloc[0, 2]

        #站点取df1，df2中非缺省的
        columns = list(sta1.columns)
        len_c1 = len(columns)
        columns_m = list(df.columns)
        for i in range(4,7):
            name1 = columns_m[i]
            name2 = columns_m[i+len_c1]
            df.loc[df[name1].isnull(), name1] = df[df[name1].isnull()][name2]

        #删除合并后第二组时空坐标信息
        drop_col = list(df.columns[len_c1:len_c1+6])
        df.drop(drop_col, axis=1, inplace=True)

        #相加前是否要先设定缺省值
        columns_m = list(df.columns)
        len_m = len(columns_m)
        if default is not None:
            for i in range(7, len_m):
                df.iloc[:, i].fillna(default, inplace=True)

        #对数据列进行相加
        len_d = len_m - len_c1
        for i in range(7,len_c1):
            df[df.columns.values[i]] = df.iloc[:, i] + df.iloc[:, i+len_d]

        #删除df2对应的数据列
        columns_drop = list(df.columns[len_c1:len_m])
        df.drop(columns_drop, axis=1, inplace=True)

        #重新命名列名称
        df.columns = columns
        return df

def max_on_id(sta1_0, sta2_0, how="left"):
    if sta1_0 is None:
        return sta2_0
    elif sta2_0 is None:
        return sta1_0
    else:
        # 删除重复行
        sta2 = sta2_0.drop_duplicates(['id'])
        sta1 = sta1_0.drop_duplicates(['id'])
        df = pd.merge(sta1, sta2, on='id', how=how)
        df.iloc[:, 0] = df.iloc[0, 0]
        df.iloc[:, 1] = df.iloc[0, 1]
        df.iloc[:, 2] = df.iloc[0, 2]
        #站点取df1，df2中非缺省的
        columns = list(sta1.columns)
        len_c1 = len(columns)
        columns_m = list(df.columns)
        for i in range(4,7):
            name1 = columns_m[i]
            name2 = columns_m[i+len_c1]
            df.loc[df[name1].isnull(), name1] = df[df[name1].isnull()][name2]

        #删除合并后第二组时空坐标信息
        drop_col = list(df.columns[len_c1:len_c1+6])
        df.drop(drop_col, axis=1, inplace=True)


        #对数据列判断最大
        datas = df.iloc[:,7:].values
        maxdata = np.max(datas,axis=1)
        df.iloc[:,i] = maxdata

        #删除df2对应的数据列
        len_m = len(df.columns)
        columns_drop = list(df.columns[8:len_m])
        df.drop(columns_drop, axis=1, inplace=True)
        #重新命名列名称
        df.columns = columns
        return df

#两个站点dataframe相减
def minus_on_id(sta1_0, sta2_0, how="left", default=None):

    # 删除重复行
    sta2 = sta2_0.drop_duplicates(['id'])
    sta1 = sta1_0.drop_duplicates(['id'])
    #将两个df1 合并在一起
    df = pd.merge(sta1, sta2, on='id', how=how)

    #时间，时效和层次，采用df1的
    df.iloc[:, 0] = df.iloc[0, 0]
    df.iloc[:, 1] = df.iloc[0, 1]
    df.iloc[:, 2] = df.iloc[0, 2]

    #站点取df1，df2中非缺省的
    columns = list(sta1.columns)
    len_c1 = len(columns)
    columns_m = list(df.columns)
    for i in range(4,7):
        name1 = columns_m[i]
        name2 = columns_m[i+len_c1]
        df.loc[df[name1].isnull(), name1] = df[df[name1].isnull()][name2]

    #删除合并后第二组时空坐标信息
    drop_col = list(df.columns[len_c1:len_c1+6])
    df.drop(drop_col, axis=1, inplace=True)

    #相加前是否要先设定缺省值
    columns_m = list(df.columns)
    len_m = len(columns_m)
    if default is not None:
        for i in range(7, len_m):
            df.iloc[:, i].fillna(default, inplace=True)

    #对数据列进行相加
    len_d = len_m - len_c1
    for i in range(7,len_c1):
        df[df.columns.values[i]] = df.iloc[:, i] - df.iloc[:, i+len_d]

    #删除df2对应的数据列
    columns_drop = list(df.columns[len_c1:len_m])
    df.drop(columns_drop, axis=1, inplace=True)

    #重新命名列名称
    df.columns = columns

    return df

#两个dataframe相乘
def multiply_on_id(sta1_0, sta2_0, how="left", default=None):
    # 删除重复行
    sta2 = sta2_0.drop_duplicates(['id'])
    sta1 = sta1_0.drop_duplicates(['id'])

    #将两个df1 合并在一起
    df = pd.merge(sta1, sta2, on='id', how=how)

    #时间，时效和层次，采用df1的
    df.iloc[:, 0] = df.iloc[0, 0]
    df.iloc[:, 1] = df.iloc[0, 1]
    df.iloc[:, 2] = df.iloc[0, 2]

    #站点取df1，df2中非缺省的
    columns = list(sta1.columns)
    len_c1 = len(columns)
    columns_m = list(df.columns)
    for i in range(4,7):
        name1 = columns_m[i]
        name2 = columns_m[i+len_c1]
        df.loc[df[name1].isnull(), name1] = df[df[name1].isnull()][name2]

    #删除合并后第二组时空坐标信息
    drop_col = list(df.columns[len_c1:len_c1+6])
    df.drop(drop_col, axis=1, inplace=True)

    #相加前是否要先设定缺省值
    columns_m = list(df.columns)
    len_m = len(columns_m)
    if default is not None:
        for i in range(7, len_m):
            df.iloc[:, i].fillna(default, inplace=True)

    #对数据列进行相加
    len_d = len_m - len_c1
    for i in range(7,len_c1):
        df[df.columns.values[i]] = df.iloc[:, i] * df.iloc[:, i+len_d]

    #删除df2对应的数据列
    columns_drop = list(df.columns[len_c1:len_m])
    df.drop(columns_drop, axis=1, inplace=True)

    #重新命名列名称
    df.columns = columns

    return df


def idw_sta_to_sta(sta0, station,effectR = 1000,nearNum = 16):
    sta1 = copy.deepcopy(station)
    xyz_sta0 = meteva.base.tool.math_tools.lon_lat_to_cartesian(sta0['lon'].values[:], sta0['lat'].values[:], R = meteva.base.basicdata.const.ER)
    xyz_sta1 = meteva.base.tool.math_tools.lon_lat_to_cartesian(sta1['lon'].values[:], sta1['lat'].values[:], R = meteva.base.basicdata.const.ER)
    tree = cKDTree(xyz_sta0)
    d, inds = tree.query(xyz_sta1, k=nearNum)
    d += 1e-6
    w = 1.0 / d ** 2
    data_name0 = meteva.base.basicdata.get_data_names(sta0)[0]
    input_dat = sta0.ix[:,data_name0].values
    dat = np.sum(w * input_dat[inds], axis=1) / np.sum(w, axis=1)
    dat[:] = np.where(d[:,0] > effectR,0,dat[:])
    data_name0 = meteva.base.basicdata.get_data_names(sta1)[0]
    sta1.ix[:,data_name0] = dat
    return sta1


def set_sta_lon_lat(sta0,station):
    # 删除重复行
    sta1 = sta0.drop_duplicates(['id'])
    station = station.drop_duplicates(['id'])
    # 先将数据合并
    column_num= len(sta1.columns)
    df = pd.merge(sta1, station, on='id', how='inner')
    sta2 = df.iloc[:,0:column_num]
    sta2.iloc[:,4:6] = df.iloc[:,column_num+4:column_num+6]
    # 重新命名列名称
    sta2.columns = sta1.columns
    return sta2

def set_sta_alt(sta0,station):
    # 删除重复行
    sta1 = sta0.drop_duplicates(['id'])
    station = station.drop_duplicates(['id'])
    # 先将数据合并
    column_num= len(sta1.columns)
    df = pd.merge(sta1, station, on='id', how='inner')
    df.loc[:,"alt_x"] = df.loc[:,"alt_y"]
    sta2 = df.iloc[:,0:column_num]
    # 重新命名列名称
    sta2.columns = sta1.columns
    return sta2

def set_sta_lon_lat_alt(sta0,station):
    # 删除重复行
    sta1 = sta0.drop_duplicates(['id'])
    station = station.drop_duplicates(['id'])
    # 先将数据合并
    column_num= len(sta1.columns)
    df = pd.merge(sta1, station, on='id', how='inner')
    sta2 = df.iloc[:,0:column_num]
    sta2.iloc[:,4:7] = df.iloc[:,column_num+4:column_num+7]
    # 重新命名列名称
    sta2.columns = sta1.columns
    return sta2

def set_value_out_9999(sta,start_value,end_value):
    data_name = meteva.base.get_data_names(sta)[0]
    sta.loc[sta[data_name] < start_value,data_name] = 9999
    sta.loc[sta[data_name] > end_value,data_name] = 9999
    return sta


# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2023， CMA National Meteorological Centre.
# All rights reserved.
#
# Distributed under the terms of the GPL v2 License.
#

import copy
import meteva_base as meb
import numpy as np
import datetime
import pandas as pd

def sample_time_select_from_sta(sta, fcst_time, fcst_fh=24, 
                                  year_before=2, day_len=60, fh_delta_list=[0,1], frt_hour=None,
                                  is_obs=False, is_show=True):
    """
    从meteva站点数据中， 筛选时间复合要求的历史训练样本并输出
    sta： meteva站点类型数据
    fcst_time(datetime.datetime), fcst_fh(int)：预报时间和时效
    year_before=2 ## 筛选往前推的年份
    day_len= 60 ## 筛选前后的日数
    fh_delta_list=[0,1] ## 筛选训练数据的预报时效，  ft = fcst_fh+fh_delta_list
    frt_hour = [8,20]## 筛选起报时间
    """ 
    import copy
    data_list = []
    for dy in np.arange(year_before+1):
        if is_obs:#实况处理(挑选)
            fcst_time0 = fcst_time + datetime.timedelta(hours=int(fcst_fh))
            fcst_fh0 = None
        else:#模式预报处理(起报时间+预报时效)
            fcst_time0 = fcst_time
            fcst_fh0 = fcst_fh

        if dy == 0:#今年
                end_time = fcst_time
                begin_time = fcst_time0 - datetime.timedelta(days=day_len)
                # print(fcst_time)
                # print(end_time, begin_time)
        else:
            end_time = copy.copy(fcst_time0).replace(year=fcst_time0.year-dy)+ datetime.timedelta(days=day_len)
            begin_time = copy.copy(fcst_time0).replace(year=fcst_time0.year-dy)- datetime.timedelta(days=day_len)
        if is_show:print("起始时间：{0}, 终止时间:{1}".format(begin_time, end_time))
        data_temp = sta.loc[(sta.time>=begin_time) & (sta.time<end_time),:]#所有历史样本数据
        if data_temp is not None:
            if  fcst_fh0 is not None:
                fhs = (fcst_fh0+ np.array(fh_delta_list))#.tolist()
                data_temp = data_temp.loc[data_temp.dtime.isin(fhs),:]
            time_index = pd.DatetimeIndex(data_temp.time)#时间索引
            if  fcst_hours is not None:
                data_temp = data_temp.loc[time_index.hour.isin(fcst_hours)]
            data_list.append(data_temp)
    data_all = pd.concat(data_list, axis=0)
    return(data_all)


def select_resemble_station(sta_info, sta_hgt, nearNum=101, alpha=200):
    """
    计算站点的相似度，返回相似站点列表
    # sta_info:所有站点，meteva的station格式
    # sta_hgt: 所有站点海拔高度
    # nearNum:邻近站点数
    # alpha: 海拔系数(距离+alpha*海拔)
    """
    from met_pre_cal.tools.temp.station_cal_nearby_select import sta_values_ensemble_nearby
    query_lonlat = sta_info.loc[:,['lon','lat']].values
    query_value = sta_info['data0'].values
    _,index,distance = sta_values_ensemble_nearby(ref_lonlat=query_lonlat, ref_value=query_value,
                                 nearNum=nearNum)
    ## id站点列表
    id_sta = sta_info.id.values
    # sta_id_邻近站点列表
    id_list = id_sta[index]
    ## 海拔高度列表
    np.set_printoptions(suppress=True)
    hgt0 = sta_hgt.data0.values
    hgt_list = hgt0[index]
    hgt0 = hgt0.reshape(-1,1)
    hgt_list = hgt_list - hgt0#海拔差delta
    ## 邻近相似度列表
    S = 1000*distance+alpha*np.abs(hgt_list)
    ## 相似度排序，由小至大返回对应id_list
    arg_S = np.argsort(S,axis=1)
    id_S_list = np.zeros_like(id_list)
    dis_S_list = np.zeros_like(distance)
    hgt_S_list = np.zeros_like(hgt_list)
    for i in np.arange(id_S_list.shape[0]):
        id_S_list[i,:] = id_list[i,:][arg_S[i,:]]
        dis_S_list[i,:] = distance[i,:][arg_S[i,:]]
        hgt_S_list[i,:] = hgt_list[i,:][arg_S[i,:]]
    return(id_S_list,dis_S_list,hgt_S_list)


def get_resemble_station_DF(sta_info, sta_hgt=None, selectNum=50):
    ## 返回相似站点id的Dataframe
    if sta_hgt is None:
        hgt = meb.read_griddata_from_nc(r"O:\jianyan\NMC\TEMP\20220323_huanan_ppt\PROData\01_nearest_topo\supply\01_HGT_1km.nc")
        sta_hgt = meb.interp_gs_linear(hgt, sta=sta_info)
    ## 相似站点选取
    id_S_list,_,_ = select_resemble_station(sta_info, sta_hgt)
    ## 邻近站点id数据
    pd_index = sta_info.id.values
    df = pd.DataFrame(id_S_list, index=pd_index, columns=np.arange(id_S_list.shape[1]))
    df = df.iloc[:,0:selectNum+1]
    return(df)


def resemble_stations_select(sta, id, resemble_id_list):
    """
    从所有数据样本中，挑选某站的相似站点对应样本,按相似度排序,并输出
    sta： meteva站点类型数据
    id:   需要计算的站点id(单站点)
    resumble_id_list: 列表，里面是所有相似站点号
    """
    ## 本站id在相似站点id的第一位
    if not id in resemble_id_list:
        resemble_id_list = np.insert(resemble_id_list, 0, id)
    ## id与排序对应表
    num0 = np.arange(len(resemble_id_list))
    rank = (dict(zip(resemble_id_list, num0)))
    ## 挑选邻近站点样本，并排序
    resemble = sta.loc[sta.id.isin(resemble_id_list),:]
    resemble['rank'] = resemble.id
    resemble['rank'] = resemble['rank'].map(rank)
    resemble   = resemble.sort_values(by='rank')
    resemble.drop(columns=['rank'],inplace=True)
    return resemble


def resample_in_resemble_stations(sta, id, resemble_id_list, 
                        value=0.5, threshold_dict={0.1:50},is_show=True):
    """
    从所有数据样本中，挑选某站的相似站点对应样本,按相似度排序,并输出
    sta： meteva站点类型数据
    id:   需要计算的站点id(单站点)
    resumble_id_list: 列表，里面是所有相似站点号
    value: 预报值
    threshold_dict: 阈值样本字典
    return: 补充总样本sta_sample，对应量级阈值，补充站点数(不包括自己)
    """
    thresholds = np.array(list(threshold_dict))
    nsamples = np.array(list(threshold_dict.values()))
    args = np.argsort(thresholds)
    thresholds = thresholds[args]
    nsamples = nsamples[args]
    if not 0 in thresholds:
        thresholds = np.insert(thresholds,0,0)
        nsamples = np.insert(nsamples,0,0)
    ## value对应具体阈值及N
    index = np.searchsorted(thresholds, value+0.01)-1
    threshold = thresholds[index]
    if index==0:
        print(value,index)
        return(None,0)
    nsample = nsamples[index]
    ## 从前向后，依次添加相似样本
    resemble_all = resemble_stations_select(sta, id, resemble_id_list)
    resemble_all['year'] = pd.DatetimeIndex(resemble_all.time).year
    years  = resemble_all['year'].drop_duplicates().sort_values(ascending=False).values
    ## 循环添加样本,判断样本数是否超过阈值
    if not 'ob' in resemble_all.columns and 'obs' in resemble_all.columns:
        resemble_all.rename(columns={'obs':'ob'},inplace=True)
    if not 'ob' in resemble_all.columns and 'data0' in resemble_all.columns:
        resemble_all.rename(columns={'data0':'ob'},inplace=True)
    resemble_list = []#样本列表
    n_flag = 0#样本数量
    for i,id_num in enumerate(resemble_id_list):
        for year in  years:
            temp  = resemble_all.loc[(resemble_all.id==id_num)&(resemble_all.year==year),:]
            temp_num =  len(temp.loc[temp['ob']>=threshold])#大于阈值的样本数
            if n_flag>=nsample:
                break
            else:
                n_flag += temp_num
                resemble_list.append(temp)
            if is_show: print("补充样本： 站点-{0}, 年份-{1},  超阈值样本数-{2}".format(id_num, year, temp_num))
        if n_flag>=nsample:
            result = pd.concat(resemble_list, axis=0)
            result.drop(columns=['year'],inplace=True)
            if is_show : print("补充样本完毕。阈值-{0}, 超阈值样本-{1}".format(threshold, len(result.loc[result['ob']>=threshold])))##样本足够
            return(result,threshold,i)
    if is_show : print("补充不足。阈值-{0}, 超阈值样本-{1}".format(threshold, len(result.loc[result['ob']>=threshold])))
    return(resemble_all.drop(columns=['year']),threshold,i)#样本不足，返回所有相似站点样本




#####  样本kalman随机上采样
def random_sample_growth(sta, threshold=0.1, k=1.1, is_show=False):
    """
    样本随机增加，大于阈值部分(实况or预报)thresold随机复制
    # method: 过采样方法- RandomOverSampler/ SMOTE/ ADASYN（随机复制或新建相似样本）
    threshold: 是否全样本kalman过采样； 为None时全样本随机取样，否则只针对实况、预报大于threshold得样本进行随机复制过采样。
    """
    # from imblearn.over_sampling import RandomOverSampler,SMOTE,ADASYN
    from collections import Counter
    col_name = 'is_pre'
    sta_t = sta.copy()
    if threshold is not None:
        temp  = (sta_t.iloc[:,-2] >= threshold) | (sta_t.iloc[:,-1]>=threshold)
        temp0 = np.zeros_like(temp, dtype=int)
        temp0[temp] = 1
        sta_t[col_name] = temp0
        sta_t = sta_t.sort_values(by=col_name)
        a = dict(sorted(Counter(sta_t[col_name].values).items()))#计数分类字典，y为0和1
        ## kalman样本过采样
        index_raw   = a.get(1)
        if index_raw is None: return sta#若无该阈值以上样本，直接返回
        index_k     = int(index_raw*k)
        sta_all     = random_over_sampler_dict(sta_t, by=col_name, target={1:index_k}, is_show=is_show)
        if is_show:
            print("Raw sample: ", dict(sorted(Counter(sta_t.loc[:, col_name]).items())))
            print("ROS sample: ", dict(sorted(Counter(sta_all.loc[:, col_name]).items())))
        sta_all.drop(columns=[col_name], inplace=True)
    else:
        index_k = int(len(sta)*k)
        sta_all = oversample_random(sta, random_num=index_k, is_show=is_show)
    return sta_all


def random_over_sampler_dict(sta,  by='is_pre', 
        target={1: 1000,  0:1000},
        is_show=False,
        ):
    """站点数据过采样， 对by数据列中的分类增加， 
        target: 字典，具体每种分类需要随机上采样至的样本数目。必须大于或等于原样本数(否则报错)
    """
    from collections import Counter
    import random

    sta = sta.sort_values(by=by)
    y = sta.loc[:, by]
    origin = dict(sorted(Counter(y).items()))#计数分类字典
    sta_all = [sta]
    for cls in target.keys():
        # sta_list = []
        if cls not in list(origin.keys()):
            raise ValueError("target class should be one of origin class")
        samples_t = target[cls]#上采样后
        samples_o = origin[cls]#上采样前
        if samples_t < samples_o:
            raise ValueError("target class samples should greater than origin class samples")
        samples_t = samples_t-samples_o
        ## 随机扩充样本
        sta_cls = sta.loc[y==cls, :]#待上采样样本
        sta_random = oversample_random(sta_cls, random_num=samples_t, is_show=is_show)
        sta_all.append(sta_random) #扩充样本入队
    ## 所有样本转为整体DataFrame
    sta_all = pd.concat(sta_all, axis=0)
    return sta_all


def oversample_random(sta, random_num, is_show=True):
    """随机上采样，random_num：随机上采样至该样本数"""
    import random
    import pandas as pd
    sta_list = []
    try:
        sta.reset_index(inplace=True, drop=True)#使用index进行增殖
        samples_t = random_num
        samples_o = len(sta)
        if is_show:
            print(r"Random Oversample: origin-{}, oversample-{}".format(samples_o, samples_t))
        ntimes = int(samples_t / samples_o)
        randoms = samples_t % samples_o
        # print(ntimes, randoms)
        if ntimes>=1:
            for ntime in range(ntimes):
                sta_list.append(sta)
        random_index   = random.sample(sta.index.values.tolist(), randoms)
        sta_random     = sta.loc[sta.index.isin(random_index),:]
        sta_list.append(sta_random)
        sta_all = pd.concat(sta_list, axis=0) #扩充样本入队
        return sta_all
    except Exception as err:
        print(err)
        return None




if __name__ == "__main__":
    sta = pd.read_hdf(r"D:\Desktop\nimm_fmm\nimm_precipitation\data\ec\01train\20230101\2023010100_train.024.h5")
    meb.set_stadata_names(sta, ['obs','fcst'])
    sta0 = meb.sele_by_para(sta, time='20201202')
    # print(sta0)
    # print((sta0.iloc[:,-2] >= 0.1) | (sta0.iloc[:,-1]>=0.1))
    # sta_t, index_k = random_sample_growth(sta0, threshold=0.1)
    # sta_all = random_over_sampler(sta_t,  by='is_pre', 
    #     target={1: 10000,  0:1000},
    #     )
    # ## test random
    # import random
    # list = [1,2,3,4,5]
    # slice = random.sample(list, 5)
    # # print(slice)
    # sta_all = oversample_random(sta_t, random_num=20000)
    # print(dict(sorted(Counter(sta_all.loc[:, 'is_pre']).items())))
    
    from FMM_base import cal_FMM_raw
    
    result=cal_FMM_raw(sta0, sta0, levels=[0])
    # sta_all = random_sample_growth(sta0, threshold=0.1, k=1.5, is_show=True)
    # print(sta0.sort_values(by='data0'))
    # print(sta_all.sort_values(by='data0'))


import ftplib
import os

    
###FTP上传####
def ftp_upload(f,file_remote,file_local,show=1):#FTP文件二进制传输
    bufsize = 1024  # 设置缓冲器大小
    with open(file_local, 'rb') as fp:
        f.storbinary('STOR ' + file_remote, fp, bufsize)
        if show:
            print("From: ",file_local)
            print("To  : ",file_remote)
            print("Upload successful!!! ")
        fp.close()    
    return()

def ftp_download(f,file_remote,file_local,show=1):#FTP文件二进制传输
    bufsize = 1024  # 设置缓冲器大小
    file_path = os.path.split(file_local)[0]
    if not os.path.exists(file_path) : os.makedirs(file_path)
    with open(file_local, 'wb') as fp:
        try:
            f.retrbinary('RETR ' + file_remote, fp.write, bufsize)
            if show:
                print("From: ",file_remote)
                print("To  : ",file_local)
                print("Download successful!!! ")
        except Exception as data:
            print("File_Download_ERROR: {0}:  {1}".format(file_local,data))
        # finally:
        fp.close()    
        return()
        

def ftp_trans(ftp_host,local_list,ftp_list,remove=False,replace=False,
                method=None, is_show=False, **kargs):
    """
    #remove:是否删除原文件，replace:是否替代目标文件
    method: mc.BF.ftp_upload为上传；mc.BF.ftp_download为下载
    """    
    if method is None: 
        method = ftp_upload

    f = ftplib.FTP()  # 实例化FTP对象
    f.connect(ftp_host[0],ftp_host[1])#ip，端口
    f.login(ftp_host[2],ftp_host[3])
    if is_show: f.set_debuglevel(2)
    f.set_pasv(True)#被动模式传输
#       f.dir()#显示ftp目录
    for i,local_file in enumerate(local_list):
        ftp_file = ftp_list[i]
        ftp_dir = os.path.split(ftp_file)[0]
#            print(ftp_dir)
        try:
            f.cwd(ftp_dir)
        except ftplib.error_perm:
            print(ftp_dir)
            f.mkd(ftp_dir)
        if replace==True:
            ftp_file_list = f.nlst(ftp_dir)
            if ftp_file in ftp_file_list: continue#原目录中有上传文件名，则不进行上传
        method(f,ftp_file,local_file,show=1)
        if remove:os.remove(local_file)
    f.quit()
    return()



###SFTP上传/下载####
def sftp_trans(sftp_host,local_list,sftp_list,method="put", remove=False):
    import paramiko
    client = paramiko.Transport( (sftp_host[0],sftp_host[1]) )
    client.connect( username=sftp_host[2],password=sftp_host[3] )
    sftp = paramiko.SFTPClient.from_transport(client)
    for i,local_file in enumerate(local_list):
        sftp_file = sftp_list[i]
        sftp_dir = os.path.split(sftp_file)[0]
        try:
            sftp.chdir(sftp_dir)
        except Exception:
            sftp.mkdir(sftp_dir)
            sftp.chdir(sftp_dir)
        finally:
            if method == "put":
                sftp.put(local_file, sftp_file)
            elif method == "get":
                sftp.get(local_file, sftp_file)
        if remove:os.remove(local_file)
    print("SFTP Success!")
    sftp.close()
    client.close()
    return()


# -*- coding: utf-8 -*-

import xarray as xr
import numpy as np
import pandas as pd
from datetime import datetime, timedelta
import copy
import meteva_base


def checkout_griddata(grd, is_single=False, valid_val=[-1000, 1000, np.nan]):
    """
    检查grd是否为符合meteva_base规定的网格数据, 未通过检查raise ValueError
    is_single: 是否检查网格数据是否为单要素(lat,lon),默认为否
    valid_val: 网格数据合理值范围下限(valid_val[0])上限(valid_val[1])， 若超过进行提示，并将超过值替换为valid_val[2]
    return:  通过检查则返回经过坐标顺序及数据类型修订后的网格数据, 数据类型强制为np.float32； 
    """

    if not isinstance(grd, xr.DataArray):
        msg = "ERROR: griddata must be xr.DataArray, please check"
        raise ValueError(msg)
    if set(grd.dims) != {'member', 'level', 'time', 'dtime', 'lat', 'lon'} :
        msg = "ERROR: griddata dims must be set of {'member', 'level', 'time', 'dtime', 'lat', 'lon'} , please check"
        raise ValueError(msg)
    if is_single:
        if len(grd.values.squeeze().shape)>2:
            msg = "ERROR: griddata has more effective coordinates than (lat, lon) , please check"
            raise ValueError(msg)
    grd0 = grd.copy()
    ## 维度顺序
    if grd0.dims != ('member', 'level', 'time', 'dtime', 'lat', 'lon'):#坐标transpose
        grd0 = grd0.transpose('member', 'level', 'time', 'dtime', 'lat', 'lon')
    ## 数据类型
    if grd0.values.dtype == np.float64:#强制数据为float32格式(后处理数据类型)
        grd0.values = grd0.values.astype(np.float32)
    ## 合理值
    if ((grd0.values<valid_val[0])|(grd0.values>valid_val[1])).any():
        msg = "WARNING: griddata values exceed VALID_VAL, setting to np.nan"
        print(msg)
        grd0.values[(grd0.values<valid_val[0])|(grd0.values>valid_val[1])] = valid_val[2]
    ## 维度数据类型
    return grd0
    

def _check_time_dtime_same(times0, dtimes0, times1, dtimes1):
    """
    time+dtime 预报时间加预报时效的结果， 两者是否相同
    input:  times-起报时间，datetime或datetime列表， dtimes-预报时效，int32或列表
    return: 相同则返回True,否则为False. 如比较24010108起报, [48,72]预报， 与24010208起报，[24,48]预报，结果为True
    """
    try:
        _ = len(times0)
    except:
        times0 = [times0]
    try:
        _ = len(dtimes0)
    except:
        dtimes0 = [dtimes0]
    try:
        _ = len(times1)
    except:
        times1 = [times1]
    try:
        _ = len(dtimes1)
    except:
        dtimes1 = [dtimes1]
    #统一为datetime格式
    times0 = [meteva_base.tool.all_type_time_to_datetime(fn) for fn in times0]
    times1 = [meteva_base.tool.all_type_time_to_datetime(fn) for fn in times1]
    ## alltime = time + dtime
    alltimes0 = []
    for time in times0:
        for dtime in dtimes0:
            alltimes0.append(time + timedelta(hours=int(dtime)))
    alltimes1 = []
    for time in times1:
        for dtime in dtimes1:
            alltimes1.append(time + timedelta(hours=int(dtime)))
    ## 两个alltime元素是否相同
    alltimes0 = set(alltimes0)
    alltimes1 = set(alltimes1)
    if alltimes0 & alltimes1 == alltimes0:
        match = True
    else:
        match = False
    return match


def checkout_griddata_same_coords(grd_list = [], is_time_match=False):
    """
    检查网格数据列表中要素是否维度信息相同或匹配(时间维度)
    is_time_match: 是否需要时间维度也可以匹配(time+dtime)，默认为True
    return: 列表网格数据维度匹配，返回True; 否则返回False
    """
    ref = grd_list[0]
    match = True
    for grd in grd_list[1:]:
        if is_time_match is False:
            match = (
                (grd.member.values == ref.member.values ).all()
                and (grd.level.values  == ref.level.values ).all()
                and np.allclose(grd.lat.values, ref.lat.values, atol=0.001)
                and np.allclose(grd.lon.values, ref.lon.values, atol=0.001)
                and match
            )
        else:
            match = (
                (grd.member.values  == ref.member.values ).all()
                and (grd.level.values  == ref.level.values ).all()
                and np.allclose(grd.lat.values, ref.lat.values, atol=0.001)
                and np.allclose(grd.lon.values, ref.lon.values, atol=0.001)
                and 
                    (((grd.time.values  == ref.time.values ).all()
                    and (grd.dtime.values  == ref.dtime.values ).all() #起报时间预报时效一致
                    ) 
                    or _check_time_dtime_same(grd.time.values, grd.dtime.values, ref.time.values, ref.dtime.values) #起报时间预报时效相加能对应
                    )   
                and match
            )
    return match


if __name__ == "__main__":
    # ## test1, time/dtime check
    # times0 = datetime(2024,3,1,8)
    # dtimes0 = [48,72,96]
    # times1 = datetime(2024,3,2,8)
    # dtimes1 = [24,48,72]
    # match = _check_time_dtime_same(times0, dtimes0, times1, dtimes1)
    # print('1:', match)
    # times2 =  [datetime(2024,3,2,8),datetime(2024,3,3,8),datetime(2024,3,4,8)]
    # dtimes2 = 24
    # match = _check_time_dtime_same(times0, dtimes0, times2, dtimes2)
    # print('2:', match)

    ## test2, time/dtime grd check
    import meteva_base as meb
    data = np.zeros((1,1,1,3,11,11),dtype=np.float64)
    grid_info0 = meb.grid(glon=[90,100,1] , glat=[30,40,1], gtime=[datetime(2024,3,1,8)], dtime_list=[48, 72, 96])
    grid_info1 = meb.grid(glon=[90,100,1] , glat=[30,40,1], gtime=[datetime(2024,3,2,8)], dtime_list=[24, 48, 96])
    grd0 = meb.grid_data(grid=grid_info0, data=data)
    grd1 = meb.grid_data(grid=grid_info1, data=data)
    # print(grd0.time.values, grd0.dtime.values, grd0.data.dtype, grd1.data.dtype)
    print(checkout_griddata_same_coords(grd_list = [grd0, grd1], is_time_match=False))
    print(checkout_griddata_same_coords(grd_list = [grd0, grd1], is_time_match=True))